import unittest

from pyhornedowl import PyIndexedOntology
from pyhornedowl.model import *
from test_base import simple_ontology, RDFS_LABEL


class IRITestCase(unittest.TestCase):

    def test_absolute_iri(self):
        o = PyIndexedOntology()
        o.prefix_mapping.add_prefix("example", "https://example.com/test#")

        expected = "https://example.com/test#A"
        self.assertEqual(expected, str(o.iri(expected)))
        self.assertEqual(expected, str(o.iri(expected, absolute=True)))

    def test_expand_curie(self):
        o = PyIndexedOntology()
        o.prefix_mapping.add_prefix("example", "https://example.com/test#")

        self.assertEqual("https://example.com/test#A", str(o.curie("example:A")))
        self.assertEqual("https://example.com/test#A", str(o.iri("example:A", absolute=False)))

    def test_expand_curie_empty_prefix(self):
        o = PyIndexedOntology()
        o.prefix_mapping.add_prefix("", "https://example.com/test#")

        self.assertEqual("https://example.com/test#A", str(o.curie(":A")))

    def test_expand_curie_unknown_prefix(self):
        o = PyIndexedOntology()

        with self.assertRaises(ValueError):
            o.curie("example:A")

    def test_iri_guess_curie(self):
        o = PyIndexedOntology()
        o.prefix_mapping.add_prefix("ex", "https://example.com/test#")

        expected = "https://example.com/test#A"

        self.assertEqual(expected, str(o.iri("ex:A", absolute=None)))
        self.assertEqual(expected, str(o.clazz("ex:A", absolute=None)))

    def test_iri_guess_absolute(self):
        o = PyIndexedOntology()
        o.prefix_mapping.add_prefix("ex", "https://example.com/test#")

        expected = "https://example.com/test#A"

        self.assertEqual(expected, str(o.iri(expected, absolute=None)))
        self.assertEqual(expected, str(o.clazz(expected, absolute=None)))

    def test_find_by_absolute(self):
        o = simple_ontology()

        expected = {AnnotatedComponent(x, set()) for x in {
            DeclareClass(o.clazz("https://example.com/A")),
            (SubClassOf(
                o.clazz("https://example.com/B"),
                o.clazz("https://example.com/A"))),
            AnnotationAssertion(o.iri("https://example.com/A"),
                                Annotation(o.annotation_property(RDFS_LABEL), SimpleLiteral("ClassA")))
        }}
        actual = o.get_axioms_for_iri("https://example.com/A", iri_is_absolute=True)

        self.assertSetEqual(expected, set(actual))

    def test_find_by_curie(self):
        o = simple_ontology()

        expected = {AnnotatedComponent(x, set()) for x in {
            DeclareClass(o.clazz("https://example.com/A")),
            (SubClassOf(
                o.clazz("https://example.com/B"),
                o.clazz("https://example.com/A"))),
            AnnotationAssertion(o.iri("https://example.com/A"),
                                Annotation(o.annotation_property(RDFS_LABEL), SimpleLiteral("ClassA")))
        }}
        actual = o.get_axioms_for_iri(":A", iri_is_absolute=False)

        self.assertSetEqual(expected, set(actual))

    def test_find_by_absolute_guess(self):
        o = simple_ontology()

        expected = {AnnotatedComponent(x, set()) for x in {
            DeclareClass(o.clazz("https://example.com/A")),
            (SubClassOf(
                o.clazz("https://example.com/B"),
                o.clazz("https://example.com/A"))),
            AnnotationAssertion(o.iri("https://example.com/A"),
                                Annotation(o.annotation_property(RDFS_LABEL), SimpleLiteral("ClassA")))
        }}
        actual = o.get_axioms_for_iri("https://example.com/A")

        self.assertSetEqual(expected, set(actual))

    def test_find_by_curie_guess(self):
        o = simple_ontology()

        expected = {AnnotatedComponent(x, set()) for x in {
            DeclareClass(o.clazz("https://example.com/A")),
            (SubClassOf(
                o.clazz("https://example.com/B"),
                o.clazz("https://example.com/A"))),
            AnnotationAssertion(o.iri("https://example.com/A"),
                                Annotation(o.annotation_property(RDFS_LABEL), SimpleLiteral("ClassA")))
        }}
        actual = o.get_axioms_for_iri(":A")

        self.assertSetEqual(expected, set(actual))


if __name__ == '__main__':
    unittest.main()



import pyhornedowl

#ontoname = "https://raw.githubusercontent.com/addicto-org/addiction-ontology/master/addicto.owx"

ontoname = "addicto.owx"

onto = pyhornedowl.open_ontology(ontoname)

clsid = onto.get_iri_for_label("tobacco smoker")

print(clsid)

#http://addictovocab.org/ADDICTO_ -> ADDICTO:
onto.add_prefix_mapping("ADDICTO","http://addictovocab.org/ADDICTO_")

clsshrtid = onto.get_id_for_iri(clsid)

print(clsshrtid)

clsid2 = onto.get_iri_for_id(clsshrtid)

print(clsid2)


clsid3 = onto.get_iri_for_id("oopsie")
print(clsid3)



import unittest

from pyhornedowl.model import *
from test_base import simple_ontology


class ModifyTestCase(unittest.TestCase):
    def test_add_collect_all_axioms(self):
        o = simple_ontology()

        axioms = set(o.get_axioms())
        axiom = EquivalentClasses([o.clazz(":A"), o.clazz(":B")])

        o.add_axiom(axiom)

        expected = {*axioms, AnnotatedComponent(axiom, set())}
        actual = set(o.get_axioms())

        self.assertSetEqual(expected, actual)

    def test_add_collect_by_iri(self):
        o = simple_ontology()

        axioms = set(o.get_axioms_for_iri(":A"))
        axiom = EquivalentClasses([o.clazz(":A"), o.clazz(":B")])

        o.add_axiom(axiom)

        expected = {*axioms, AnnotatedComponent(axiom, set())}
        actual = set(o.get_axioms_for_iri(":A"))

        self.assertSetEqual(expected, actual)

    def test_remove_collect_all_axioms(self):
        o = simple_ontology()

        axioms = set(o.get_axioms())
        axiom = SubClassOf(o.clazz(":B"), o.clazz(":A"))

        removed = o.remove_axiom(axiom)
        self.assertTrue(removed, "Axiom was not removed!")

        expected = axioms - {AnnotatedComponent(axiom, set())}
        actual = set(o.get_axioms())

        self.assertSetEqual(expected, actual)

    def test_remove_collect_by_iri(self):
        o = simple_ontology()

        all_axioms = set(o.get_axioms())
        axioms = set(o.get_components_for_iri(":A"))
        axiom = SubClassOf(o.clazz(":B"), o.clazz(":A"))
        ann_axiom = AnnotatedComponent(axiom, set())

        removed = o.remove_axiom(axiom)
        
        self.assertTrue(removed, "Axiom was not removed!")
        
        expected = all_axioms - {ann_axiom}
        actual = set(o.get_axioms())
        self.assertSetEqual(expected, actual, "Axiom not removed from all axioms")

        self.skipTest("Removing axioms from the IRI index is not done properly in HornedOWL at the moment. https://github.com/phillord/horned-owl/pull/121")
        expected = axioms - {ann_axiom}
        actual = set(o.get_components_for_iri(":A"))
        self.assertSetEqual(expected, actual, "Axiom not removed from IRI index")

    def test_add_component_with_set_annos(self):
        o = simple_ontology()

        axiom = SubClassOf(o.clazz(":B"), o.clazz(":A"))

        annos0 = {Annotation(o.annotation_property(":a"), SimpleLiteral("Test"))}
        ann_axiom0 = AnnotatedComponent(axiom, annos0)
        o.add_axiom(axiom, annos0)
        assert(ann_axiom0 in o.get_axioms())

        annos1 = {Annotation(o.annotation_property(":a"), SimpleLiteral("Test"))}
        ann_axiom1 = AnnotatedComponent(axiom, annos1)
        o.add_axiom(axiom, annos1)
        assert(ann_axiom1 in o.get_axioms())

        annos2 = {
            Annotation(o.annotation_property(":a"), SimpleLiteral("Test1")),
            Annotation(o.annotation_property(":b"), SimpleLiteral("Test2")),
        }
        ann_axiom2 = AnnotatedComponent(axiom, annos2)
        o.add_axiom(axiom, annos2)
        assert(ann_axiom2 in o.get_axioms())

    def test_add_component_with_list_annos(self):
        o = simple_ontology()

        axiom = SubClassOf(o.clazz(":B"), o.clazz(":A"))

        annos0 = [Annotation(o.annotation_property(":a"), SimpleLiteral("Test"))]
        ann_axiom0 = AnnotatedComponent(axiom, set(annos0))
        o.add_axiom(axiom, annos0)
        assert(ann_axiom0 in o.get_axioms())

        annos1 = [Annotation(o.annotation_property(":a"), SimpleLiteral("Test"))]
        ann_axiom1 = AnnotatedComponent(axiom, set(annos1))
        o.add_axiom(axiom, annos1)
        assert(ann_axiom1 in o.get_axioms())

        annos2 = [
            Annotation(o.annotation_property(":a"), SimpleLiteral("Test1")),
            Annotation(o.annotation_property(":b"), SimpleLiteral("Test2")),
        ]
        ann_axiom2 = AnnotatedComponent(axiom, set(annos2))
        o.add_axiom(axiom, annos2)
        assert(ann_axiom2 in o.get_axioms())






if __name__ == '__main__':
    unittest.main()



from ..pyhornedowl import model
import typing

Class = model.Class
ObjectIntersectionOf = model.ObjectIntersectionOf
ObjectUnionOf = model.ObjectUnionOf
ObjectComplementOf = model.ObjectComplementOf
ObjectOneOf = model.ObjectOneOf
ObjectSomeValuesFrom = model.ObjectSomeValuesFrom
ObjectAllValuesFrom = model.ObjectAllValuesFrom
ObjectHasValue = model.ObjectHasValue
ObjectHasSelf = model.ObjectHasSelf
ObjectMinCardinality = model.ObjectMinCardinality
ObjectMaxCardinality = model.ObjectMaxCardinality
ObjectExactCardinality = model.ObjectExactCardinality
DataSomeValuesFrom = model.DataSomeValuesFrom
DataAllValuesFrom = model.DataAllValuesFrom
DataHasValue = model.DataHasValue
DataMinCardinality = model.DataMinCardinality
DataMaxCardinality = model.DataMaxCardinality
DataExactCardinality = model.DataExactCardinality
Datatype = model.Datatype
DataIntersectionOf = model.DataIntersectionOf
DataUnionOf = model.DataUnionOf
DataComplementOf = model.DataComplementOf
DataOneOf = model.DataOneOf
DatatypeRestriction = model.DatatypeRestriction
SimpleLiteral = model.SimpleLiteral
LanguageLiteral = model.LanguageLiteral
DatatypeLiteral = model.DatatypeLiteral
ObjectProperty = model.ObjectProperty
InverseObjectProperty = model.InverseObjectProperty
AnnotatedComponent = model.AnnotatedComponent
Annotation = model.Annotation
AnnotationAssertion = model.AnnotationAssertion
AnnotationProperty = model.AnnotationProperty
AnnotationPropertyDomain = model.AnnotationPropertyDomain
AnnotationPropertyRange = model.AnnotationPropertyRange
AnonymousIndividual = model.AnonymousIndividual
AsymmetricObjectProperty = model.AsymmetricObjectProperty
ClassAssertion = model.ClassAssertion
DataProperty = model.DataProperty
DataPropertyAssertion = model.DataPropertyAssertion
DataPropertyDomain = model.DataPropertyDomain
DataPropertyRange = model.DataPropertyRange
DatatypeDefinition = model.DatatypeDefinition
DeclareAnnotationProperty = model.DeclareAnnotationProperty
DeclareClass = model.DeclareClass
DeclareDataProperty = model.DeclareDataProperty
DeclareDatatype = model.DeclareDatatype
DeclareNamedIndividual = model.DeclareNamedIndividual
DeclareObjectProperty = model.DeclareObjectProperty
DifferentIndividuals = model.DifferentIndividuals
DisjointClasses = model.DisjointClasses
DisjointDataProperties = model.DisjointDataProperties
DisjointObjectProperties = model.DisjointObjectProperties
DisjointUnion = model.DisjointUnion
EquivalentClasses = model.EquivalentClasses
EquivalentDataProperties = model.EquivalentDataProperties
EquivalentObjectProperties = model.EquivalentObjectProperties
FacetRestriction = model.FacetRestriction
FunctionalDataProperty = model.FunctionalDataProperty
FunctionalObjectProperty = model.FunctionalObjectProperty
HasKey = model.HasKey
IRI = model.IRI
Import = model.Import
InverseFunctionalObjectProperty = model.InverseFunctionalObjectProperty
InverseObjectProperties = model.InverseObjectProperties
IrreflexiveObjectProperty = model.IrreflexiveObjectProperty
NamedIndividual = model.NamedIndividual
NegativeDataPropertyAssertion = model.NegativeDataPropertyAssertion
NegativeObjectPropertyAssertion = model.NegativeObjectPropertyAssertion
ObjectPropertyAssertion = model.ObjectPropertyAssertion
ObjectPropertyDomain = model.ObjectPropertyDomain
ObjectPropertyRange = model.ObjectPropertyRange
OntologyAnnotation = model.OntologyAnnotation
ReflexiveObjectProperty = model.ReflexiveObjectProperty
SameIndividual = model.SameIndividual
SubAnnotationPropertyOf = model.SubAnnotationPropertyOf
SubClassOf = model.SubClassOf
SubDataPropertyOf = model.SubDataPropertyOf
SubObjectPropertyOf = model.SubObjectPropertyOf
SymmetricObjectProperty = model.SymmetricObjectProperty
TransitiveObjectProperty = model.TransitiveObjectProperty
OntologyID = model.OntologyID
DocIRI = model.DocIRI
Rule = model.Rule
Variable = model.Variable
BuiltInAtom = model.BuiltInAtom
ClassAtom = model.ClassAtom
DataPropertyAtom = model.DataPropertyAtom
DataRangeAtom = model.DataRangeAtom
DifferentIndividualsAtom = model.DifferentIndividualsAtom
ObjectPropertyAtom = model.ObjectPropertyAtom
SameIndividualAtom = model.SameIndividualAtom
Facet = model.Facet
ClassExpression = model.ClassExpression
ObjectPropertyExpression = model.ObjectPropertyExpression
SubObjectPropertyExpression = model.SubObjectPropertyExpression
Literal = model.Literal
DataRange = model.DataRange
Individual = model.Individual
PropertyExpression = model.PropertyExpression
AnnotationSubject = model.AnnotationSubject
AnnotationValue = model.AnnotationValue
Component = model.Component
Atom = model.Atom
IArgument = model.IArgument
DArgument = model.DArgument

__all__ = ['Class', 'ObjectIntersectionOf', 'ObjectUnionOf', 'ObjectComplementOf', 'ObjectOneOf', 'ObjectSomeValuesFrom', 'ObjectAllValuesFrom', 'ObjectHasValue', 'ObjectHasSelf', 'ObjectMinCardinality', 'ObjectMaxCardinality', 'ObjectExactCardinality', 'DataSomeValuesFrom', 'DataAllValuesFrom', 'DataHasValue', 'DataMinCardinality', 'DataMaxCardinality', 'DataExactCardinality', 'Datatype', 'DataIntersectionOf', 'DataUnionOf', 'DataComplementOf', 'DataOneOf', 'DatatypeRestriction', 'SimpleLiteral', 'LanguageLiteral', 'DatatypeLiteral', 'ObjectProperty', 'InverseObjectProperty', 'AnnotatedComponent', 'Annotation', 'AnnotationAssertion', 'AnnotationProperty', 'AnnotationPropertyDomain', 'AnnotationPropertyRange', 'AnonymousIndividual', 'AsymmetricObjectProperty', 'ClassAssertion', 'DataProperty', 'DataPropertyAssertion', 'DataPropertyDomain', 'DataPropertyRange', 'DatatypeDefinition', 'DeclareAnnotationProperty', 'DeclareClass', 'DeclareDataProperty', 'DeclareDatatype', 'DeclareNamedIndividual', 'DeclareObjectProperty', 'DifferentIndividuals', 'DisjointClasses', 'DisjointDataProperties', 'DisjointObjectProperties', 'DisjointUnion', 'EquivalentClasses', 'EquivalentDataProperties', 'EquivalentObjectProperties', 'FacetRestriction', 'FunctionalDataProperty', 'FunctionalObjectProperty', 'HasKey', 'IRI', 'Import', 'InverseFunctionalObjectProperty', 'InverseObjectProperties', 'IrreflexiveObjectProperty', 'NamedIndividual', 'NegativeDataPropertyAssertion', 'NegativeObjectPropertyAssertion', 'ObjectPropertyAssertion', 'ObjectPropertyDomain', 'ObjectPropertyRange', 'OntologyAnnotation', 'ReflexiveObjectProperty', 'SameIndividual', 'SubAnnotationPropertyOf', 'SubClassOf', 'SubDataPropertyOf', 'SubObjectPropertyOf', 'SymmetricObjectProperty', 'TransitiveObjectProperty', 'OntologyID', 'DocIRI', 'Rule', 'Variable', 'BuiltInAtom', 'ClassAtom', 'DataPropertyAtom', 'DataRangeAtom', 'DifferentIndividualsAtom', 'ObjectPropertyAtom', 'SameIndividualAtom', 'Facet', 'ClassExpression', 'ObjectPropertyExpression', 'SubObjectPropertyExpression', 'Literal', 'DataRange', 'Individual', 'PropertyExpression', 'AnnotationSubject', 'AnnotationValue', 'Component', 'Atom', 'IArgument', 'DArgument']


import time

import pyhornedowl
from pyhornedowl.model import *

print("Loading ontology")

ontoname = "chebi_core.owl"

FULL = True
RDFSLABEL = "http://www.w3.org/2000/01/rdf-schema#label"
SMILES = "http://purl.obolibrary.org/obo/chebi/smiles"
DEFN = "http://purl.obolibrary.org/obo/IAO_0000115"
SYN = "http://purl.obolibrary.org/obo/IAO_0000115"
HAS_ROLE = "http://purl.obolibrary.org/obo/RO_0000087"
HAS_PART = "http://purl.obolibrary.org/obo/BFO_0000051"
CHARGE = "http://purl.obolibrary.org/obo/chebi/charge"

start = time.time_ns()
onto = pyhornedowl.open_ontology(ontoname)
end = time.time_ns()
print(f"Loaded {ontoname} in {end - start}ns")

print (f"Loaded ontology has {len(onto.get_classes())} classes.")
print (f"Loaded ontology has {len(onto.get_axioms())} axioms.")

#print("Sleeping")
#time.sleep(5)

clssname = "caffeine"
clssid: str = onto.get_iri_for_label(clssname)

print(f"{clssname} has iri {clssid}")

print(f"Class {clssid} ({clssname}) has axioms: {onto.get_axioms_for_iri(clssid)}")


onto.add_axiom(SubClassOf(Class(onto.iri("https://test.test/test")), Class(  onto.iri('http://testing.testing'))))
onto.add_axiom(AnnotationAssertion(onto.iri('http://test_annotation'), Annotation(AnnotationProperty( onto.iri('http://purl.obolibrary.org/obo/CHEBI_27732')), SimpleLiteral('Testing Added Annotation Value'))))
print (f"Ontology now has {len(onto.get_axioms())} axioms.")


print ("Removing one axiom")
onto.remove_axiom(SubClassOf(Class(onto.iri("https://test.test/test")), Class(  onto.iri('http://testing.testing'))))
print (f"Ontology now has {len(onto.get_axioms())} axioms.")

print ("Adding a complex axiom")

# onto.add_axiom(['SubClassOf', 'http://purl.obolibrary.org/obo/CHEBI_27732', ['ObjectSomeValuesFrom', 'http://test-op', 'http://test-ob-target']])
onto.add_axiom(SubClassOf(Class(onto.iri("http://purl.obolibrary.org/obo/CHEBI_27732")), ObjectSomeValuesFrom(ObjectProperty(onto.iri('http://test-op')), Class(onto.iri('http://test-ob-target')))))

# onto.add_axiom(['SubClassOf', 'http://purl.obolibrary.org/obo/CHEBI_27732', ['ObjectIntersectionOf', 'http://purl.obolibrary.org/obo/CHEBI_27134', 'http://test-ob-target']])
onto.add_axiom(SubClassOf(Class(onto.iri("http://purl.obolibrary.org/obo/CHEBI_27732")), ObjectIntersectionOf([Class(onto.iri('http://purl.obolibrary.org/obo/CHEBI_27134')), Class(onto.iri('http://test-ob-target'))])))

print (f"Ontology now has {len(onto.get_axioms())} axioms.")

print (f"Getting all classes with charge and no smiles")
# Get all classes with asserted charge but no smiles
if FULL:
    classes_with_charge = set()
    classes_with_smiles = set()
    for a in onto.get_axioms():
        #Example: ['AnnotationAssertion', 'http://purl.obolibrary.org/obo/CHEBI_27732', 'http://www.geneontology.org/formats/oboInOwl#hasAlternativeId', 'CHEBI:41472']
        # if len(a)==4 and a[0]=='AnnotationAssertion':
        if isinstance(a.axiom, AnnotationAssertion):

            # if a[2]==CHARGE:
            if a.axiom.ann.ap == onto.iri(CHARGE):

                # chg = a[3]
                chg = a.axiom.ann.av
                # classes_with_charge.add(a[1])
                classes_with_charge.add(a.axiom.subject)

            # if a[2]==SMILES:
            if a.axiom.ann.ap == onto.iri(SMILES):

                # smls = a[3]
                smls = a.axiom.ann.av

                # classes_with_smiles.add(a[1])
                classes_with_smiles.add(a.axiom.subject)

    classes_with_charge_no_smiles = [c for c in classes_with_charge if c not in classes_with_smiles]
    print(f"The ontology has {len(classes_with_charge_no_smiles)} classes with charge and no smiles.")

# Get all the asserted parts:
if FULL:
    print("Getting all asserted part relations")
    asserted_parts = set() # A list of tuples
    for a in onto.get_axioms():
        # Example: ['SubClassOf', 'http://purl.obolibrary.org/obo/CHEBI_27732', ['ObjectSomeValuesFrom', 'http://purl.obolibrary.org/obo/RO_0000087', 'http://purl.obolibrary.org/obo/CHEBI_85234']]
        # if len(a)==3 and a[0]=='SubClassOf' and \
        #   isinstance(a[2], list) and len(a[2])==3 and \
        #   a[2][0]=='ObjectSomeValuesFrom' and a[2][1]==HAS_PART:
        if isinstance(a.axiom, SubClassOf) and isinstance(a.axiom.sup, ObjectSomeValuesFrom) \
            and a.axiom.sup.ope == onto.iri(HAS_PART):
            # asserted_parts.add((a[1],a[2][2]))
            asserted_parts.add((a.axiom.sub, a.axiom.sup.bce))


    print(f"There are {len(asserted_parts)} parthood relations asserted in ChEBI.")

#print("Sleeping")
#time.sleep(5)

#print("Axioms for IRI ",clssid,": ",onto.get_axioms_for_iri(clssid))

if FULL:
    for input_label in ["carboxy group","codeine","caffeine"]:
        clssid = onto.get_iri_for_label(input_label)
        output_label = onto.get_annotation(clssid,RDFSLABEL)
        print(f"Got label '{output_label}' for iri {clssid}")
        output_def = onto.get_annotation(clssid,DEFN)
        print(f"Got definition '{output_def}' for iri {clssid}")
        output_sm = onto.get_annotation(clssid,SMILES)
        print(f"Got SMILES '{output_sm}' for iri {clssid}")
        output_ax = onto.get_axioms_for_iri(clssid)
        print(f"Got axioms for iri {clssid}: {output_ax}")


#print("Sleeping")
#time.sleep(5)

if FULL:
    clssid = onto.get_iri_for_label("caffeine")
    ancestors = pyhornedowl.get_ancestors(onto,clssid)
    print(f"Got ancestors {ancestors} for class {clssid}")
    print("Getting a whole lot of labels for IRIs")
    descs = pyhornedowl.get_descendants(onto,onto.get_iri_for_label('carboxylic acid'))
    desc_names = [onto.get_annotation(d,RDFSLABEL) for d in descs]
    print(len(desc_names))

if FULL:
    newlabel = "caffeine2"
    onto.set_label(clssid,newlabel)
    print(f"The new label of the class {clssid} is '{onto.get_annotation(clssid,RDFSLABEL)}'")

#print("Sleeping")
#time.sleep(5)

# Currently this is a bit slow. But it does work!
#if FULL:
#    onto.save_to_file("test/chebi-updated.owx")

print("Quitting")






#Memory Profiler for all your memory needs.

#https://pypi.python.org/pypi/memory_profiler

#Run a pip install:

#pip install memory_profiler

#Import the library:

#import memory_profiler

#Add a decorator to the item you wish to profile:

#@profile
#def my_func():
#    a = [1] * (10 ** 6)
#    b = [2] * (2 * 10 ** 7)
#    del b
#    return a

#if __name__ == '__main__':
#    my_func()

#Execute the code:

#python -m memory_profiler example.py

#Recieve the output:

# Line #    Mem usage  Increment   Line Contents
# ==============================================
# 3                           @profile
# 4      5.97 MB    0.00 MB   def my_func():
# 5     13.61 MB    7.64 MB       a = [1] * (10 ** 6)
# 6    166.20 MB  152.59 MB       b = [2] * (2 * 10 ** 7)
# 7     13.61 MB -152.59 MB       del b
# 8     13.61 MB    0.00 MB       return a

#Examples are from the docs, linked above.



import json
import os
from typing import Optional, Literal

from jinja2 import Environment, select_autoescape, FileSystemLoader, pass_context

REPO_ROOT=os.path.join(os.path.dirname(__file__), "..")

def as_py_type(val, module: Optional[str] = None):
    if isinstance(val, dict) and "type" in val:
        typ = val["type"]
        if "inner" in val:
            inner_ = val['inner']
            if typ == "Vec":
                return f"typing.List[{as_py_type(inner_)}]"
            if typ == "Box":
                return as_py_type(inner_)
            if typ == "BTreeSet":
                return f"typing.Set[{as_py_type(inner_)}]"
            if typ == "Option":
                return f"typing.Optional[{as_py_type(inner_)}]"

        return typ if module is None else f"{module}.{typ}"
    elif isinstance(val, list):
        return f"typing.Tuple[{','.join(as_py_type(x) for x in val)}]"
    else:
        return {
            "u32": "int",
            "String": "str",
            "StringWrapper": "str"
        }.get(val, val if module is None else f"{module}.{val}")


def as_rust_type(val):
    wrapped_types = {
        "Vec": "VecWrap",
        "Box": "BoxWrap",
        "BTreeSet": "BTreeSetWrap"
    }
    if isinstance(val, dict) and "type" in val:
        typ = val["type"]

        if "inner" in val:
            inner_ = val['inner']
            outer = wrapped_types.get(typ, typ)
            return f"{outer}<{as_rust_type(inner_)}>"
        else:
            return wrapped_types.get(typ, typ)
    elif isinstance(val, list):
        return f"({','.join(as_rust_type(x) for x in val)})"
    else:
        return wrapped_types.get(val, val)


def as_py_name(val):
    return {
        0: "first",
        1: "second",
        2: "third",
        3: "fourth",
        4: "fifth",
        5: "sixth",
        6: "seventh",
        7: "eighth",
        8: "ninth",
        9: "tenth",
    }.get(val, val)


def py_field(field, type):
    if isinstance(type, dict):
        return type.get("py_name", field)

    return field


@pass_context
def f_rust(ctx, val):
    if "variant" in ctx:
        fields = ctx["variant"]["fields"]
    else:
        fields = ctx["model"]["fields"]

    if isinstance(val, tuple) and isinstance(fields, dict):
        return val[0]
    if isinstance(val, str) and isinstance(fields, dict):
        return val
    if isinstance(val, list) and "loop" in ctx:
        return ctx["loop"]["index0"]

    return val


def fields(val):
    if isinstance(val, dict):
        return val.items()
    if isinstance(val, list):
        return list(enumerate(val))

    return val

def build_from_templates(lang: Literal["rs", "pyi"]):
    with open(os.path.join(REPO_ROOT, "template", "model.json")) as f:
        data = json.load(f)

    env = Environment(
        loader=FileSystemLoader(os.path.join(REPO_ROOT, "template", "templates")),
        autoescape=select_autoescape()
    )

    env.filters['as_py_type'] = as_py_type
    env.filters['as_rust_type'] = as_rust_type
    env.filters["as_py_name"] = as_py_name
    env.filters["py_field"] = py_field
    env.filters["fields"] = fields
    env.filters["f_rust"] = f_rust
    env.tests['list'] = lambda value: isinstance(value, list)

    out = []

    header_template = env.get_template(f"static.{lang}")
    out.append(header_template.render(models=data) + "\n\n")

    for model in data:
        type = model["type"]
        template = env.get_template(f"{type}.{lang}.jinja2")
        res = template.render(model=model)
        out.append(res + "\n")

    return "".join(out)

def main():

    with open(os.path.join(REPO_ROOT, "src", "model_generated.rs"), "w") as f:
        f.write(build_from_templates("rs"))


if __name__ == "__main__":
    main()



import pyhornedowl
import time

from pyhornedowl.model import SubClassOf


def perf(path: str):
    print(f"===={path}====")
    print(f"Opening...")

    open_time= 0
    for i in range(3):
        start = time.perf_counter()
        o = pyhornedowl.open_ontology(path)
        open_time += time.perf_counter() - start

    open_time /= 3
    print(f"Opened in {open_time}s")

    print("Iterating...")
    amount = 0
    iter_time = 0
    for _ in range(3):
        start = time.perf_counter()
        for a in o.get_components():
            if len(a.ann) > 0:
                amount += 1
        iter_time += time.perf_counter() - start

    iter_time /= 3
    print(f"Iterated in {iter_time}s")

    print("Adding 100.000 axioms...")
    o = pyhornedowl.PyIndexedOntology()
    start = time.perf_counter()
    for i in range(100_000):
        o.add_axiom(SubClassOf(o.clazz(f"https://example.com/entity_{i}"), o.clazz(f"https://example.com/entity{i+1}")))
    add_time = time.perf_counter() - start
    print(f"Added 100.000 axioms in {add_time}s")


if __name__ == "__main__":
    perf("go.owl")
    perf("chebi_core.owl")


import unittest

from test_base import simple_ontology


class LabelTestCase(unittest.TestCase):
    def test_label_for_iri_existing(self):
        o = simple_ontology()

        expected = "https://example.com/A"
        actual = o.get_iri_for_label("ClassA")

        self.assertEqual(expected, actual)

    def test_label_for_iri_not_existing(self):
        o = simple_ontology()

        expected = None
        actual = o.get_iri_for_label("ClassZ")

        self.assertEqual(expected, actual)

    def test_update_label(self):
        o = simple_ontology()

        expected = "NewClassA"
        o.set_label(":A", expected)

        actual = o.get_annotation(":A", "rdfs:label")

        self.assertEqual(expected, actual)

        all_labels = o.get_annotations(":A", "rdfs:label")
        self.assertEqual([expected], all_labels)

    def test_add_label(self):
        o = simple_ontology()

        expected = "ClassC"
        o.set_label(":C", expected)

        actual = o.get_annotation(":C", "rdfs:label")

        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



import re
from typing import List

def process_signature(app, what, name, obj, options, signature: str, return_annotation):
    if signature is not None:
        signature = re.sub(r"BoxWrap<(.*)>", r"\1", signature)
        signature = signature.replace("<", "[")
        signature = signature.replace(">", "]")
        signature = signature.replace("VecWrap", "typing.List")
        signature = signature.replace("StringWrap", "str")
        signature = signature.replace("BTreeSetWrap", "typing.Set")
        signature = signature.replace("u32", "int")

    if return_annotation is not None:
        return_annotation = re.sub(r"BoxWrap<(.*)>", r"\1", return_annotation)
        return_annotation = return_annotation.replace("<", "[")
        return_annotation = return_annotation.replace(">", "]")
        return_annotation = return_annotation.replace("VecWrap", "typing.List")
        return_annotation = return_annotation.replace("StringWrap", "str")
        return_annotation = return_annotation.replace("BTreeSetWrap", "typing.Set")
        return_annotation = return_annotation.replace("u32", "int")

    return (signature, return_annotation)

def process_docstring(app, what, name, obj, options, lines: List[str]):
    text = "\n".join(lines)
    md_code = re.compile(r"(`[^`]+`)")
    md_internal_link = re.compile(r"\[([^][]+)\]\(struct\.([^()]+)\.html\)")
    md_link = re.compile(r'\[([^][]+)\]\(([^()]+)\)')

    text = re.sub(md_code, r":code:\1", text)
    text = re.sub(md_internal_link, r":py:class:`~pyhornedowl.model.\2`", text)
    text = re.sub(md_link, r"`\1 <\2>`_", text)

    lines.clear()
    lines += text.splitlines()
            

def setup(app):
    app.connect('autodoc-process-signature', process_signature)
    app.connect('autodoc-process-docstring', process_docstring)



import unittest

import pyhornedowl
from test_base import simple_ontology


class IdTestCase(unittest.TestCase):
    def test_id_from_iri_empty(self):
        o = simple_ontology()
        o.prefix_mapping.add_prefix("ex", "https://example.com/")

        expected = "ex:A"
        actual = o.get_id_for_iri("https://example.com/A")

        # The empty prefix was inserted earlier, hence it will be used to define the ID
        self.assertNotEqual(expected, actual)

    def test_id_from_absolute(self):
        o = pyhornedowl.PyIndexedOntology()
        o.prefix_mapping.add_prefix("EX", "https://example.com/")

        expected = "EX:A"
        actual = o.get_id_for_iri("https://example.com/A")

        self.assertEqual(expected, actual)

    def test_id_from_curie_empty_prefix(self):
        o = pyhornedowl.PyIndexedOntology()
        o.prefix_mapping.add_prefix("", "https://example.com/")

        expected = "A"
        actual = o.get_id_for_iri(":A")

        self.assertEqual(expected, actual)

    def test_id_from_curie_defined_prefix(self):
        o = pyhornedowl.PyIndexedOntology()
        o.prefix_mapping.add_prefix("EX", "https://example.com/")

        expected = "EX:A"
        actual = o.get_id_for_iri("EX:A")

        self.assertEqual(expected, actual)

    def test_iri_from_id(self):
        o = simple_ontology()
        o.prefix_mapping.add_prefix("ex", "https://example.com/")

        expected = "https://example.com/A"
        actual = o.get_iri_for_id("ex:A")

        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



import pyhornedowl
from pyhornedowl.model import *

#ontoname = "https://raw.githubusercontent.com/addicto-org/addiction-ontology/master/addicto.owx"

ontoname = "test/addicto-merged.owx"

HAS_PART = "http://purl.obolibrary.org/obo/BFO_0000051"
PARTICIPATES_IN = "http://purl.obolibrary.org/obo/RO_0000056"

onto = pyhornedowl.open_ontology(ontoname)

clsid:str = onto.get_iri_for_label("tobacco smoker")

print(clsid)

clsid2:str = onto.get_iri_for_label("cigarette smoking")
print(clsid2)

print(f"Ontology has {len(onto.get_axioms())} axioms.")
# onto.add_axiom(['EquivalentClasses', clsid, ['ObjectSomeValuesFrom',PARTICIPATES_IN,clsid2]])
onto.add_axiom(EquivalentClasses([Class(onto.iri(clsid)), ObjectSomeValuesFrom(ObjectProperty(onto.iri(PARTICIPATES_IN)), Class(onto.iri(clsid2)))]))
print(f"Ontology has {len(onto.get_axioms())} axioms.")

clsid3:str = onto.get_iri_for_label("abstainer")

# onto.add_axiom(['DisjointClasses', clsid, clsid3])
onto.add_axiom(DisjointClasses([Class(onto.iri(clsid)), Class(onto.iri(clsid3))]))
print(f"Ontology has {len(onto.get_axioms())} axioms.")



import typing
import unittest
from tempfile import NamedTemporaryFile

import pyhornedowl
from test_base import r, res, simple_ontology

SERIALIZATIONS: typing.List[typing.Literal['ofn', 'owx', 'owl']] = ['ofn', 'owx', 'owl']


class IOTestCase(unittest.TestCase):

    def assertOntologiesEqual(self, actual, expected):
        self.assertSetEqual(set(expected.get_axioms()), set(actual.get_axioms()), "Axioms do not match!")
        self.assertEqual(expected.get_iri(), actual.get_iri(), "Ontology IRIs do not match!")

    def test_load_simple_from_string_generic_guess(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                content = res(f"simple.{s}")

                actual = pyhornedowl.open_ontology(content)
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    def test_load_simple_from_string_generic_explicit(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                content = res(f"simple.{s}")

                actual = pyhornedowl.open_ontology(content, s)
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    def test_load_simple_from_string_explicit_wrong_serialization(self):
        content = res(f"simple.owl")
        with self.assertRaises(ValueError):
            pyhornedowl.open_ontology(content, "something")

        with self.assertRaises(TypeError):
            pyhornedowl.open_ontology(content, 42)

    @unittest.skip("Horned parsers currently panic")
    def test_load_simple_from_string_implicit_fail(self):
        content = "Lorem ipsum dolor sit amet"
        with self.assertRaises(ValueError):
            o = pyhornedowl.open_ontology(content)
            pass

    def test_load_simple_from_string_explicit(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                content = res(f"simple.{s}")

                actual = pyhornedowl.open_ontology_from_string(content, s)
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    @unittest.skip("Functional syntax parser parses others formats without errors others panic")
    def test_load_simple_from_string_guess_parser(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                content = res(f"simple.{s}")

                actual = pyhornedowl.open_ontology_from_string(content)
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    def test_load_simple_from_file_generic_guess(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                actual = pyhornedowl.open_ontology(r(f'simple.{s}'))
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    def test_load_simple_from_file_generic_explicit(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                actual = pyhornedowl.open_ontology(r(f'simple.{s}'), s)
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    def test_load_simple_from_file_guess_ext(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                actual = pyhornedowl.open_ontology_from_file(r(f'simple.{s}'))
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    def test_load_simple_from_file_explicit(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                actual = pyhornedowl.open_ontology_from_file(r(f'simple.{s}.raw'), s)
                expected = simple_ontology()

                self.assertOntologiesEqual(actual, expected)

    def test_write_simple_guess_ext(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                with NamedTemporaryFile(suffix=f".{s}") as f:
                    original = simple_ontology()
                    original.prefix_mapping.remove_prefix("")
                    original.save_to_file(f.name)

                    actual = pyhornedowl.open_ontology_from_file(f.name, s)

                    self.assertOntologiesEqual(original, actual)

    def test_write_simple_explicit(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                with NamedTemporaryFile(suffix=f".{s}.raw") as f:
                    original = simple_ontology()
                    original.prefix_mapping.remove_prefix("")
                    original.save_to_file(f.name, s)

                    actual = pyhornedowl.open_ontology_from_file(f.name, s)

                    self.assertOntologiesEqual(original, actual)
                    
    def test_write_simple_to_string(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                original = simple_ontology()
                original.prefix_mapping.remove_prefix("")
                o = original.save_to_string(s)

                actual = pyhornedowl.open_ontology_from_string(o, s)

                self.assertOntologiesEqual(original, actual)


    def test_prefix(self):
        for s in SERIALIZATIONS:
            with self.subTest(serialization=s):
                if s == 'owl':
                    self.skipTest('RDF/XML parser does not return prefixes')

                o = pyhornedowl.open_ontology(r(f'prefix.{s}'), s)

                actual = set(iter(o.prefix_mapping))
                expected = set({"": "https://example.com/",
                                "ex1": "https://example.com/1",
                                "ex2": "https://example.com/2",
                                "ex3": "https://example.com/3",
                                "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
                                "xsd": "http://www.w3.org/2001/XMLSchema#",
                                "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
                                "owl": "http://www.w3.org/2002/07/owl#",
                                "xml": "http://www.w3.org/XML/1998/namespace",
                                }.items())

                self.assertSetEqual(expected, actual)

if __name__ == '__main__':
    unittest.main()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'py-horned-owl'
copyright = '2024, Janna Hastings, Björn Gehrke'
author = 'Janna Hastings, Björn Gehrke'
release = '0.3.1'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration
# import pathlib
# import sys
# sys.path.insert(0, pathlib.Path(__file__).parents[2].resolve().as_posix())

import sys
import os
sys.path.append(os.path.dirname(__file__))

import custom_doc

autosummary_imported_members = True
autodoc_default_options = {
    'members': True,
    'special-members': '__init__',
}
autodoc_typehints = "both"

autosummary_generate = True

autodoc_type_aliases = {
    'ClassExpression ': 'pyhornedowl.module.ClassExpression',
}

extensions = [
    'sphinx.ext.autodoc',
   'sphinx.ext.autosummary',
   "custom_doc"
]

templates_path = ['_templates']
exclude_patterns = []



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']



import unittest

from test_base import simple_ontology


class HierarchyTestCase(unittest.TestCase):
    def test_no_subclass(self):
        o = simple_ontology()

        expected = set()
        actual = o.get_subclasses(":C")

        self.assertSetEqual(expected, actual)

    def test_no_superclass(self):
        o = simple_ontology()

        expected = set()
        actual = o.get_superclasses(":C")

        self.assertSetEqual(expected, actual)

    def test_direct_subclass(self):
        o = simple_ontology()

        expected = {"https://example.com/B"}
        actual = o.get_subclasses(":A")

        self.assertSetEqual(expected, actual)

    def test_direct_superclass(self):
        o = simple_ontology()

        expected = {"https://example.com/A"}
        actual = o.get_superclasses(":B")

        self.assertSetEqual(expected, actual)

    def test_no_ancestors(self):
        o = simple_ontology()

        expected = set()
        actual = o.get_ancestors(":A")

        self.assertSetEqual(expected, actual)

    def test_no_descendants(self):
        o = simple_ontology()

        expected = set()
        actual = o.get_descendants(":C")

        self.assertSetEqual(expected, actual)

    def test_single_ancestors(self):
        o = simple_ontology()

        expected = {"https://example.com/A"}
        actual = o.get_ancestors(":B")

        self.assertSetEqual(expected, actual)

    def test_single_descendants(self):
        o = simple_ontology()

        expected = {"https://example.com/D"}
        actual = o.get_descendants(":B")

        self.assertSetEqual(expected, actual)

    def test_multiple_ancestors(self):
        o = simple_ontology()

        expected = {"https://example.com/A", "https://example.com/B"}
        actual = o.get_ancestors(":D")

        self.assertSetEqual(expected, actual)

    def test_multiple_descendants(self):
        o = simple_ontology()

        expected = {"https://example.com/B", "https://example.com/D"}
        actual = o.get_descendants(":A")

        self.assertSetEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



import unittest

import pyhornedowl
from pyhornedowl import IndexCreationStrategy
from pyhornedowl.model import *
from test_base import r, RDFS_LABEL


class IndexTestCase(unittest.TestCase):

    def test_default_build_on_query(self):
        o = pyhornedowl.open_ontology_from_file(r(f'simple.owl'))
        actual = o.get_axioms_for_iri('https://example.com/A')

        self.assertSetEqual({AnnotatedComponent(a, set()) for a in {
            DeclareClass(o.clazz("https://example.com/A")),
            SubClassOf(
                o.clazz("https://example.com/B"),
                o.clazz("https://example.com/A")),
            AnnotationAssertion(
                o.iri("https://example.com/A"),
                Annotation(AnnotationProperty(o.iri(RDFS_LABEL)),
                           SimpleLiteral("ClassA"))),
        }}, set(actual))

    def test_explicit_fail(self):
        o = pyhornedowl.open_ontology_from_file(r(f'simple.owl'), index_strategy=IndexCreationStrategy.Explicit)

        with self.assertRaises(ValueError):
            o.get_axioms_for_iri("https://example.com/A")

    def test_explicit(self):
        o = pyhornedowl.open_ontology_from_file(r(f'simple.owl'), index_strategy=IndexCreationStrategy.Explicit)

        o.build_indexes()

        actual = o.get_axioms_for_iri("https://example.com/A")

        self.assertSetEqual({AnnotatedComponent(a, set()) for a in {
            DeclareClass(o.clazz("https://example.com/A")),
            SubClassOf(
                o.clazz("https://example.com/B"),
                o.clazz("https://example.com/A")),
            AnnotationAssertion(
                o.iri("https://example.com/A"),
                Annotation(AnnotationProperty(o.iri(RDFS_LABEL)),
                           SimpleLiteral("ClassA"))),
        }}, set(actual))

    def test_on_load(self):
        o = pyhornedowl.open_ontology_from_file(r(f'simple.owl'), index_strategy=IndexCreationStrategy.OnLoad)

        actual = o.get_axioms_for_iri("https://example.com/A")

        self.assertSetEqual({AnnotatedComponent(a, set()) for a in {
            DeclareClass(o.clazz("https://example.com/A")),
            SubClassOf(
                o.clazz("https://example.com/B"),
                o.clazz("https://example.com/A")),
            AnnotationAssertion(
                o.iri("https://example.com/A"),
                Annotation(AnnotationProperty(o.iri(RDFS_LABEL)),
                           SimpleLiteral("ClassA"))),
        }}, set(actual))


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations
from .pyhornedowl import PyIndexedOntology, IndexCreationStrategy, PrefixMapping, open_ontology, open_ontology_from_file, open_ontology_from_string, get_descendants, get_ancestors, create_reasoner, PyReasoner

__all__ = ["PyIndexedOntology", "IndexCreationStrategy", "PrefixMapping", "open_ontology", "open_ontology_from_file", "open_ontology_from_string", "get_descendants", "get_ancestors", "create_reasoner", "PyReasoner"]



# Extracts the documentation from horned owl to attach it to the python types

import json
import os
import subprocess
from os import environ

REPO_ROOT=os.path.join(os.path.dirname(__file__), "..")
# path to local horned owl copy
HORNED_OWL_PATH = environ.get("HORNED_OWL_REPO_PATH", os.path.join(REPO_ROOT, "..", "horned-owl"))

subprocess.call("cargo +nightly rustdoc -Z unstable-options --output-format json".split(" "), cwd=HORNED_OWL_PATH)

with open(os.path.join(HORNED_OWL_PATH, "target", "doc", "horned_owl.json")) as f:
    data = json.load(f)

docs = dict([(x['name'], x['docs']) for x in data["index"].values() if
             x['span'] is not None and x['span']['filename'] == "src/model.rs" and
             x['name'] is not None and
             x['docs'] is not None and
             ('struct' in x['inner'] or 'enum' in x['inner'] or 'variant' in x['inner'])])

lines = [f'    ({n}) => {{ {json.dumps(d)} }};' for n, d in docs.items()]
lines.sort()
lines.append("    ($t:ident) => {\"\"}")

out_path = os.path.join(REPO_ROOT, "src", "doc.rs")
with open(out_path, "w") as f:
    f.write('macro_rules! doc (\n' + "\n".join(lines) + '\n);\n')



import unittest

import pyhornedowl


class PrefixTestCase(unittest.TestCase):

    def test_missing_prefix(self):
        o = pyhornedowl.PyIndexedOntology()

        with self.assertRaises(ValueError) as context:
            c = o.clazz("ex:A")

        self.assertEqual(context.exception.args[0], "Invalid curie: Invalid")

    def test_prefix(self):
        o = pyhornedowl.PyIndexedOntology()
        o.prefix_mapping.add_prefix("ex", "https://example.com/")

        c = o.clazz("ex:A")

        self.assertEqual(str(c), "https://example.com/A")

    def test_default_prefix_fail(self):
        o = pyhornedowl.PyIndexedOntology()

        with self.assertRaises(ValueError) as context:
            c = o.clazz("A")

        self.assertEqual(context.exception.args[0], "Invalid curie: MissingDefault")

    def test_default_prefix(self):
        o = pyhornedowl.PyIndexedOntology()
        o.prefix_mapping.add_prefix("", "https://example.com/")

        c = o.clazz("A")

        self.assertEqual(str(c), "https://example.com/A")


if __name__ == '__main__':
    unittest.main()



import typing
from typing import *

class IRI:
    parse: Any
    ...

class Facet:
    Length: Facet
    MinLength: Facet
    MaxLength: Facet
    Pattern: Facet
    MinInclusive: Facet
    MinExclusive: Facet
    MaxInclusive: Facet
    MaxExclusive: Facet
    TotalDigits: Facet
    FractionDigits: Facet
    LangRange: Facet


import unittest

from pyhornedowl import PyIndexedOntology
from pyhornedowl.model import *


class ClassExpressionBuilderTestCase(unittest.TestCase):
    def test_ce_not(self):
        o = PyIndexedOntology()
        c = o.clazz("https://example.com/A")

        expected = ObjectComplementOf(c)
        actual = ~c

        self.assertEqual(expected, actual)

    def test_ce_not_complex(self):
        o = PyIndexedOntology()
        c = ObjectUnionOf([o.clazz("https://example.com/A"), o.clazz("https://example.com/B")])

        expected = ObjectComplementOf(c)
        actual = ~c

        self.assertEqual(expected, actual)

    def test_ce_and(self):
        o = PyIndexedOntology()
        c1 = o.clazz("https://example.com/A")
        c2 = o.clazz("https://example.com/B")

        expected = ObjectIntersectionOf([c1, c2])
        actual = c1 & c2

        self.assertEqual(expected, actual)

    def test_ce_or(self):
        o = PyIndexedOntology()
        c1 = o.clazz("https://example.com/A")
        c2 = o.clazz("https://example.com/B")

        expected = ObjectUnionOf([c1, c2])
        actual = c1 | c2

        self.assertEqual(expected, actual)

    def test_op_not(self):
        o = PyIndexedOntology()
        op = o.object_property("https://example.com/op")

        expected = InverseObjectProperty(op)
        actual = ~op

        self.assertEqual(expected, actual)

    def test_ce_some(self):
        o = PyIndexedOntology()
        c = o.clazz("https://example.com/A")
        op = o.object_property("https://example.com/op")

        expected = ObjectSomeValuesFrom(op, c)
        actual = op.some(c)

        self.assertEqual(expected, actual)

    def test_ce_only(self):
        o = PyIndexedOntology()
        c = o.clazz("https://example.com/A")
        op = o.object_property("https://example.com/op")

        expected = ObjectAllValuesFrom(op, c)
        actual = op.only(c)

        self.assertEqual(expected, actual)

    def test_ce_min(self):
        o = PyIndexedOntology()
        c = o.clazz("https://example.com/A")
        op = o.object_property("https://example.com/op")

        expected = ObjectMinCardinality(2, op, c)
        actual = op.min(2, c)

        self.assertEqual(expected, actual)

    def test_ce_max(self):
        o = PyIndexedOntology()
        c = o.clazz("https://example.com/A")
        op = o.object_property("https://example.com/op")

        expected = ObjectMaxCardinality(2, op, c)
        actual = op.max(2, c)

        self.assertEqual(expected, actual)

    def test_ce_exact(self):
        o = PyIndexedOntology()
        c = o.clazz("https://example.com/A")
        op = o.object_property("https://example.com/op")

        expected = ObjectExactCardinality(2, op, c)
        actual = op.exact(2, c)

        self.assertEqual(expected, actual)

    def test_ce_has_value(self):
        o = PyIndexedOntology()
        op = o.object_property("https://example.com/op")
        i = o.named_individual("https://example.com/I")

        expected = ObjectHasValue(op, i)
        actual = op.has_value(i)

        self.assertEqual(expected, actual)

    def test_ce_has_self(self):
        o = PyIndexedOntology()
        op = o.object_property("https://example.com/op")

        expected = ObjectHasSelf(op)
        actual = op.has_self()

        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



import glob
import inspect
import json
import os
import re
import typing

from build_model import as_py_type, as_py_name, build_from_templates


REPO_ROOT=os.path.join(os.path.dirname(__file__), "..")
_CWD = os.getcwd()
os.chdir(REPO_ROOT)


old_files = glob.glob("pyhornedowl/**/__init__.*", recursive=True)

for f in old_files:
    os.unlink(f)

import pyhornedowl.pyhornedowl as pho

os.makedirs("pyhornedowl/model", exist_ok=True)

with open("pyhornedowl/__init__.py", "w") as f:
    pyhornedowl_members = [k for k, v in pho.__dict__.items() if isinstance(v, type) or callable(v)]

    f.write("from __future__ import annotations\n")
    f.write("from .pyhornedowl import ")
    f.write(", ".join(pyhornedowl_members))

    f.write("\n\n")
    f.write("__all__ = [")
    f.write(", ".join([f'"{n}"' for n in pyhornedowl_members]))
    f.write("]\n")

implemented_magic = [f"__{x}__" for x in
                     ["invert", "and", "or", "invert", "iter", "getitem", "setitem", "contains", "len", "delitem"]]

with open("pyhornedowl/__init__.pyi", "w") as f:
    f.write("import typing\n")
    f.write("from typing import *\n")
    f.write("from typing_extensions import deprecated\n\n")
    f.write("import model\n")
    f.write("\n")

    for name, entry in pho.__dict__.items():
        if isinstance(entry, type):
            f.write(f"class {name}:\n")
            # There appears to be a bug with pyo3. Documentation on enum 
            # variants is not attached to their mapped python types. Hence we 
            # use a workarround of adding their documentation to the enum in
            # the style: "<MemberName>: <doc string>".
            member_docs = {}
            if hasattr(entry, "__doc__"):
                entry_doc = entry.__doc__
                if entry_doc is not None:
                    f.write("    \"\"\"\n")
                    for line in entry_doc.splitlines():
                        member_doc_m = re.match(r"^(\w+): (.*)$", line)
                        if member_doc_m:
                            member_docs[member_doc_m.group(1)]=member_doc_m.group(2)
                        else:
                            f.write(f"    {line}\n")

                    f.write("    \"\"\"\n")

            for member_name, member in entry.__dict__.items():
                if member_name.startswith("_") and member_name not in implemented_magic:
                    continue
                
                # E.g. for enums
                if isinstance(member, entry):
                    f.write(f"    {member_name}: typing.Self\n")
                    if member_name in member_docs or hasattr(member, "__doc__") and member.__doc__ is not None:
                        doc = member_docs.get(member_name, getattr(member, "__doc__"))
                        f.write("    \"\"\"\n")
                        for line in doc.splitlines():
                            f.write(f"    {line}\n")
                        f.write("    \"\"\"\n")

                    continue

                if hasattr(member, "__doc__"):
                    doc = member.__doc__
                    if doc is not None:
                        lines = doc.splitlines()
                        if len(lines) > 2:
                            annotations_end = lines.index(next(x for x in lines if not x.startswith("@")), 0)
                            annotations = lines[:annotations_end]
                            sign = lines[annotations_end] 

                            for ann in annotations:
                                f.write(f"    {ann}\n")

                            if callable(member):
                                f.write(f"    def {sign}:\n")
                                doc = "\n".join([f"        {l}" for l in lines[annotations_end+2:]])
                                f.write(f'        """\n{doc}\n        """\n        ...\n\n')
                            else:
                                f.write(f"    {sign}\n")
                                doc = "\n".join([f"    {l}" for l in lines[annotations_end+2:]])
                                f.write(f'    """\n{doc}\n    """\n\n')

                            continue

                if callable(member):
                    f.write(f"    def {member_name}{inspect.signature(member)}:\n        ...\n\n")

                    continue
                                
                    
            f.write("\n")
        elif callable(entry):
            if hasattr(entry, "__doc__"):
                doc = entry.__doc__
                if doc is not None:
                    lines = doc.splitlines()
                    if len(lines) > 2:
                        annotations_end = lines.index(next(x for x in lines if not x.startswith("@")), 0)
                        annotations = lines[:annotations_end]
                        sign = lines[annotations_end] 

                        for ann in annotations:
                            f.write(f"{ann}\n")
                        f.write(f"def {sign}:\n")
                        doc = "\n".join([f"    {l}" for l in lines[annotations_end+2:]])
                        f.write(f'    """\n{doc}\n    """\n    ...\n\n')

            f.write("\n")

with open("pyhornedowl/py.typed", "w"):
    pass


def handle_module(module: str):
    with open(f"pyhornedowl/{module}/__init__.py", "w") as f:
        f.write(f"from ..pyhornedowl import {module}\n")
        f.write("import typing\n\n")

        al = []

        for name, entry in getattr(pho, module).__dict__.items():
            if not isinstance(entry, type) and not type(entry) == typing._UnionGenericAlias:
                continue

            f.write(f"{name} = {module}.{name}\n")
            al.append(name)

        f.write("\n")

        f.write(f"__all__ = {al}")

    with open(f"pyhornedowl/{module}/__init__.pyi", "w") as f:
        f.write(build_from_templates("pyi"))
        # f.write("import typing\n")
        # f.write("from typing import *\n")
        #
        #
        #
        # for name, entry in getattr(pho, module).__dict__.items():
        #     if isinstance(entry, type):
        #         f.write(f"class {name}:\n")
        #         for attr_name, attr in entry.__dict__.items():
        #             if attr_name.startswith("_") and attr_name not in implemented_magic:
        #                 continue
        #
        #             doc = attr.__doc__ if hasattr(attr, "__doc__") else ""
        #             doc = None if doc == "" else doc
        #             if callable(attr):
        #                 if attr_name.startswith("__"):
        #                     f.write(f"    def {attr_name}{str(inspect.signature(attr))}:\n")
        #             else:
        #                 f.write(f"    {doc if doc else f"{attr_name}: Any"}\n")
        #
        #         f.write("    ...\n")
        #     elif type(entry) == typing._UnionGenericAlias:
        #         f.write(f"{name} = {str(entry).replace(f'pyhornedowl.{module}.', '')}")
        #     else:
        #         continue
        #
        #     f.write("\n")
        #
        # f.write("\n")


handle_module("model")

os.chdir(_CWD)



import os

from typing import List

import pyhornedowl
from pyhornedowl.model import *

RDFS_LABEL = "http://www.w3.org/2000/01/rdf-schema#label"


def r(*args: str) -> str:
    return os.path.abspath(os.path.join(os.path.dirname(__file__), "resources", *args))


def res(resource: str) -> str:
    with open(r(resource)) as f:
        return f.read()


def simple_ontology_comps() -> List[Component]:
    return [
        DeclareClass(Class(IRI.parse("https://example.com/A"))),
        DeclareClass(Class(IRI.parse("https://example.com/B"))),
        DeclareClass(Class(IRI.parse("https://example.com/C"))),
        DeclareClass(Class(IRI.parse("https://example.com/D"))),
        SubClassOf(
            sup=Class(IRI.parse("https://example.com/A")),
            sub=Class(IRI.parse("https://example.com/B"))),
        SubClassOf(
            sup=Class(IRI.parse("https://example.com/B")),
            sub=Class(IRI.parse("https://example.com/D"))),
        AnnotationAssertion(
            IRI.parse("https://example.com/A"),
            Annotation(AnnotationProperty(IRI.parse(RDFS_LABEL)),
                       SimpleLiteral("ClassA"))),
        AnnotationAssertion(
            IRI.parse("https://example.com/B"),
            Annotation(AnnotationProperty(IRI.parse(RDFS_LABEL)),
                       SimpleLiteral("ClassB")))
    ]


def simple_ontology_comps_annotated() -> List[AnnotatedComponent]:
    return [AnnotatedComponent(x, set()) for x in simple_ontology_comps()]


def simple_ontology() -> pyhornedowl.PyIndexedOntology:
    onto = pyhornedowl.PyIndexedOntology()
    onto.prefix_mapping.add_default_prefix_names()
    onto.prefix_mapping.add_prefix("", "https://example.com/")

    for c in simple_ontology_comps():
        onto.add_component(c)

    return onto



import typing
from typing import *

class IRI:
    parse: Any
    ...

class Facet:
    Length: Facet
    MinLength: Facet
    MaxLength: Facet
    Pattern: Facet
    MinInclusive: Facet
    MinExclusive: Facet
    MaxInclusive: Facet
    MaxExclusive: Facet
    TotalDigits: Facet
    FractionDigits: Facet
    LangRange: Facet

class Class:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    def __str__(self) -> str:
        ...
    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...

class AnonymousIndividual:
    first: str

    def __init__(self,first: str,/):
        ...


    def __str__(self) -> str:
        ...
    ...

class NamedIndividual:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    def __str__(self) -> str:
        ...
    ...

class ObjectProperty:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    def __str__(self) -> str:
        ...
    def some(self, ce: ClassExpression) -> ObjectSomeValuesFrom:
        """Existentional relationship"""
        ...

    def only(self, ce: ClassExpression) -> ObjectAllValuesFrom:
        """Universal relationship"""
        ...

    def has_value(self, individual: Individual) -> ObjectHasValue:
        """Existential relationship to an individual"""
        ...

    def has_self(self) -> ObjectHasSelf:
        """Individuals with relation to themselves"""
        ...

    def min(self, n: int, ce: ClassExpression) -> ObjectMinCardinality:
        """Minimum cardinality relationship"""
        ...

    def max(self, n: int, ce: ClassExpression) -> ObjectMaxCardinality:
        """Maximum cardinality relationship"""
        ...

    def exact(self, n: int, ce: ClassExpression) -> ObjectExactCardinality:
        """Exact cardinality relationship"""
        ...

    def __invert__(self) -> ObjectPropertyExpression:
        """Inverse of object property expression"""
        ...
    ...

class Datatype:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    def __str__(self) -> str:
        ...
    ...

class DataProperty:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    def __str__(self) -> str:
        ...
    ...

class FacetRestriction:
    f: Facet
    l: Literal

    def __init__(self,f: Facet,l: Literal,/):
        ...

    ...

Individual = typing.Union[AnonymousIndividual,NamedIndividual,]


class InverseObjectProperty:
    first: ObjectProperty

    def __init__(self,first: ObjectProperty,):
        ...
    

    def some(self, ce: ClassExpression) -> ObjectSomeValuesFrom:
        """Existentional relationship"""
        ...

    def only(self, ce: ClassExpression) -> ObjectAllValuesFrom:
        """Universal relationship"""
        ...

    def has_value(self, individual: Individual) -> ObjectHasValue:
        """Existential relationship to an individual"""
        ...

    def has_self(self) -> ObjectHasSelf:
        """Individuals with relation to themselves"""
        ...

    def min(self, n: int, ce: ClassExpression) -> ObjectMinCardinality:
        """Minimum cardinality relationship"""
        ...

    def max(self, n: int, ce: ClassExpression) -> ObjectMaxCardinality:
        """Maximum cardinality relationship"""
        ...

    def exact(self, n: int, ce: ClassExpression) -> ObjectExactCardinality:
        """Exact cardinality relationship"""
        ...

    def __invert__(self) -> ObjectPropertyExpression:
        """Inverse of object property expression"""
        ...
    ...

ObjectPropertyExpression = typing.Union[ObjectProperty,InverseObjectProperty,]


class SimpleLiteral:
    literal: str

    def __init__(self,literal: str,):
        ...
    

    ...
class LanguageLiteral:
    literal: str
    lang: str

    def __init__(self,literal: str,lang: str,):
        ...
    

    ...
class DatatypeLiteral:
    literal: str
    datatype_iri: IRI

    def __init__(self,literal: str,datatype_iri: IRI,):
        ...
    

    ...

Literal = typing.Union[SimpleLiteral,LanguageLiteral,DatatypeLiteral,]


class DataIntersectionOf:
    first: typing.List[DataRange]

    def __init__(self,first: typing.List[DataRange],):
        ...
    

    ...
class DataUnionOf:
    first: typing.List[DataRange]

    def __init__(self,first: typing.List[DataRange],):
        ...
    

    ...
class DataComplementOf:
    first: DataRange

    def __init__(self,first: DataRange,):
        ...
    

    ...
class DataOneOf:
    first: typing.List[Literal]

    def __init__(self,first: typing.List[Literal],):
        ...
    

    ...
class DatatypeRestriction:
    first: Datatype
    second: typing.List[FacetRestriction]

    def __init__(self,first: Datatype,second: typing.List[FacetRestriction],):
        ...
    

    ...

DataRange = typing.Union[Datatype,DataIntersectionOf,DataUnionOf,DataComplementOf,DataOneOf,DatatypeRestriction,]


class ObjectIntersectionOf:
    first: typing.List[ClassExpression]

    def __init__(self,first: typing.List[ClassExpression],):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectUnionOf:
    first: typing.List[ClassExpression]

    def __init__(self,first: typing.List[ClassExpression],):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectComplementOf:
    first: ClassExpression

    def __init__(self,first: ClassExpression,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectOneOf:
    first: typing.List[Individual]

    def __init__(self,first: typing.List[Individual],):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectSomeValuesFrom:
    ope: ObjectPropertyExpression
    bce: ClassExpression

    def __init__(self,ope: ObjectPropertyExpression,bce: ClassExpression,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectAllValuesFrom:
    ope: ObjectPropertyExpression
    bce: ClassExpression

    def __init__(self,ope: ObjectPropertyExpression,bce: ClassExpression,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectHasValue:
    ope: ObjectPropertyExpression
    i: Individual

    def __init__(self,ope: ObjectPropertyExpression,i: Individual,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectHasSelf:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectMinCardinality:
    n: int
    ope: ObjectPropertyExpression
    bce: ClassExpression

    def __init__(self,n: int,ope: ObjectPropertyExpression,bce: ClassExpression,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectMaxCardinality:
    n: int
    ope: ObjectPropertyExpression
    bce: ClassExpression

    def __init__(self,n: int,ope: ObjectPropertyExpression,bce: ClassExpression,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class ObjectExactCardinality:
    n: int
    ope: ObjectPropertyExpression
    bce: ClassExpression

    def __init__(self,n: int,ope: ObjectPropertyExpression,bce: ClassExpression,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class DataSomeValuesFrom:
    dp: DataProperty
    dr: DataRange

    def __init__(self,dp: DataProperty,dr: DataRange,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class DataAllValuesFrom:
    dp: DataProperty
    dr: DataRange

    def __init__(self,dp: DataProperty,dr: DataRange,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class DataHasValue:
    dp: DataProperty
    l: Literal

    def __init__(self,dp: DataProperty,l: Literal,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class DataMinCardinality:
    n: int
    dp: DataProperty
    dr: DataRange

    def __init__(self,n: int,dp: DataProperty,dr: DataRange,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class DataMaxCardinality:
    n: int
    dp: DataProperty
    dr: DataRange

    def __init__(self,n: int,dp: DataProperty,dr: DataRange,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...
class DataExactCardinality:
    n: int
    dp: DataProperty
    dr: DataRange

    def __init__(self,n: int,dp: DataProperty,dr: DataRange,):
        ...
    

    def __and__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Intersection of two class expressions"""
        ...

    def __or__(self, ce: ClassExpression) -> ObjectIntersectionOf:
        """Union of two class expressions"""
        ...

    def __invert__(self) -> ObjectIntersectionOf:
        """Complement of a class expression"""
        ...
    ...

ClassExpression = typing.Union[Class,ObjectIntersectionOf,ObjectUnionOf,ObjectComplementOf,ObjectOneOf,ObjectSomeValuesFrom,ObjectAllValuesFrom,ObjectHasValue,ObjectHasSelf,ObjectMinCardinality,ObjectMaxCardinality,ObjectExactCardinality,DataSomeValuesFrom,DataAllValuesFrom,DataHasValue,DataMinCardinality,DataMaxCardinality,DataExactCardinality,]

PropertyExpression = typing.Union[ObjectPropertyExpression,DataProperty,AnnotationProperty,]

AnnotationSubject = typing.Union[IRI,AnonymousIndividual,]

class AnnotationProperty:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    ...

AnnotationValue = typing.Union[Literal,IRI,AnonymousIndividual,]

class Annotation:
    ap: AnnotationProperty
    av: AnnotationValue

    def __init__(self,ap: AnnotationProperty,av: AnnotationValue,/):
        ...

    ...

class OntologyAnnotation:
    first: Annotation

    def __init__(self,first: Annotation,/):
        ...


    ...

class Import:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    ...

class DeclareClass:
    first: Class

    def __init__(self,first: Class,/):
        ...


    ...

class DeclareObjectProperty:
    first: ObjectProperty

    def __init__(self,first: ObjectProperty,/):
        ...


    ...

class DeclareAnnotationProperty:
    first: AnnotationProperty

    def __init__(self,first: AnnotationProperty,/):
        ...


    ...

class DeclareDataProperty:
    first: DataProperty

    def __init__(self,first: DataProperty,/):
        ...


    ...

class DeclareNamedIndividual:
    first: NamedIndividual

    def __init__(self,first: NamedIndividual,/):
        ...


    ...

class DeclareDatatype:
    first: Datatype

    def __init__(self,first: Datatype,/):
        ...


    ...

class SubClassOf:
    sub: ClassExpression
    sup: ClassExpression

    def __init__(self,sub: ClassExpression,sup: ClassExpression,/):
        ...

    ...

class EquivalentClasses:
    first: typing.List[ClassExpression]

    def __init__(self,first: typing.List[ClassExpression],/):
        ...


    ...

class DisjointClasses:
    first: typing.List[ClassExpression]

    def __init__(self,first: typing.List[ClassExpression],/):
        ...


    ...

class DisjointUnion:
    first: Class
    second: typing.List[ClassExpression]

    def __init__(self,first: Class,second: typing.List[ClassExpression],/):
        ...


    ...

SubObjectPropertyExpression = typing.Union[typing.List[ObjectPropertyExpression],ObjectPropertyExpression,]

class SubObjectPropertyOf:
    sub: SubObjectPropertyExpression
    sup: ObjectPropertyExpression

    def __init__(self,sub: SubObjectPropertyExpression,sup: ObjectPropertyExpression,/):
        ...

    ...

class EquivalentObjectProperties:
    first: typing.List[ObjectPropertyExpression]

    def __init__(self,first: typing.List[ObjectPropertyExpression],/):
        ...


    ...

class DisjointObjectProperties:
    first: typing.List[ObjectPropertyExpression]

    def __init__(self,first: typing.List[ObjectPropertyExpression],/):
        ...


    ...

class InverseObjectProperties:
    first: ObjectProperty
    second: ObjectProperty

    def __init__(self,first: ObjectProperty,second: ObjectProperty,/):
        ...


    ...

class ObjectPropertyDomain:
    ope: ObjectPropertyExpression
    ce: ClassExpression

    def __init__(self,ope: ObjectPropertyExpression,ce: ClassExpression,/):
        ...

    ...

class ObjectPropertyRange:
    ope: ObjectPropertyExpression
    ce: ClassExpression

    def __init__(self,ope: ObjectPropertyExpression,ce: ClassExpression,/):
        ...

    ...

class FunctionalObjectProperty:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,/):
        ...


    ...

class InverseFunctionalObjectProperty:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,/):
        ...


    ...

class ReflexiveObjectProperty:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,/):
        ...


    ...

class IrreflexiveObjectProperty:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,/):
        ...


    ...

class SymmetricObjectProperty:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,/):
        ...


    ...

class AsymmetricObjectProperty:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,/):
        ...


    ...

class TransitiveObjectProperty:
    first: ObjectPropertyExpression

    def __init__(self,first: ObjectPropertyExpression,/):
        ...


    ...

class SubDataPropertyOf:
    sub: DataProperty
    sup: DataProperty

    def __init__(self,sub: DataProperty,sup: DataProperty,/):
        ...

    ...

class EquivalentDataProperties:
    first: typing.List[DataProperty]

    def __init__(self,first: typing.List[DataProperty],/):
        ...


    ...

class DisjointDataProperties:
    first: typing.List[DataProperty]

    def __init__(self,first: typing.List[DataProperty],/):
        ...


    ...

class DataPropertyDomain:
    dp: DataProperty
    ce: ClassExpression

    def __init__(self,dp: DataProperty,ce: ClassExpression,/):
        ...

    ...

class DataPropertyRange:
    dp: DataProperty
    dr: DataRange

    def __init__(self,dp: DataProperty,dr: DataRange,/):
        ...

    ...

class FunctionalDataProperty:
    first: DataProperty

    def __init__(self,first: DataProperty,/):
        ...


    ...

class DatatypeDefinition:
    kind: Datatype
    range: DataRange

    def __init__(self,kind: Datatype,range: DataRange,/):
        ...

    ...

class HasKey:
    ce: ClassExpression
    vpe: typing.List[PropertyExpression]

    def __init__(self,ce: ClassExpression,vpe: typing.List[PropertyExpression],/):
        ...

    ...

class SameIndividual:
    first: typing.List[Individual]

    def __init__(self,first: typing.List[Individual],/):
        ...


    ...

class DifferentIndividuals:
    first: typing.List[Individual]

    def __init__(self,first: typing.List[Individual],/):
        ...


    ...

class ClassAssertion:
    ce: ClassExpression
    i: Individual

    def __init__(self,ce: ClassExpression,i: Individual,/):
        ...

    ...

class ObjectPropertyAssertion:
    ope: ObjectPropertyExpression
    source: Individual
    target: Individual

    def __init__(self,ope: ObjectPropertyExpression,source: Individual,target: Individual,/):
        ...

    ...

class NegativeObjectPropertyAssertion:
    ope: ObjectPropertyExpression
    source: Individual
    target: Individual

    def __init__(self,ope: ObjectPropertyExpression,source: Individual,target: Individual,/):
        ...

    ...

class DataPropertyAssertion:
    dp: DataProperty
    source: Individual
    target: Literal

    def __init__(self,dp: DataProperty,source: Individual,target: Literal,/):
        ...

    ...

class NegativeDataPropertyAssertion:
    dp: DataProperty
    source: Individual
    target: Literal

    def __init__(self,dp: DataProperty,source: Individual,target: Literal,/):
        ...

    ...

class AnnotationAssertion:
    subject: AnnotationSubject
    ann: Annotation

    def __init__(self,subject: AnnotationSubject,ann: Annotation,/):
        ...

    ...

class SubAnnotationPropertyOf:
    sub: AnnotationProperty
    sup: AnnotationProperty

    def __init__(self,sub: AnnotationProperty,sup: AnnotationProperty,/):
        ...

    ...

class AnnotationPropertyDomain:
    ap: AnnotationProperty
    iri: IRI

    def __init__(self,ap: AnnotationProperty,iri: IRI,/):
        ...

    ...

class AnnotationPropertyRange:
    ap: AnnotationProperty
    iri: IRI

    def __init__(self,ap: AnnotationProperty,iri: IRI,/):
        ...

    ...

class DocIRI:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    ...

class OntologyID:
    iri: typing.Optional[IRI]
    viri: typing.Optional[IRI]

    def __init__(self,iri: typing.Optional[IRI],viri: typing.Optional[IRI],/):
        ...

    ...

class Variable:
    first: IRI

    def __init__(self,first: IRI,/):
        ...


    ...

DArgument = typing.Union[Literal,Variable,]

IArgument = typing.Union[Individual,Variable,]


class BuiltInAtom:
    pred: IRI
    args: typing.List[DArgument]

    def __init__(self,pred: IRI,args: typing.List[DArgument],):
        ...
    

    ...
class ClassAtom:
    pred: ClassExpression
    arg: IArgument

    def __init__(self,pred: ClassExpression,arg: IArgument,):
        ...
    

    ...
class DataPropertyAtom:
    pred: DataProperty
    args: typing.Tuple[DArgument,DArgument]

    def __init__(self,pred: DataProperty,args: typing.Tuple[DArgument,DArgument],):
        ...
    

    ...
class DataRangeAtom:
    pred: DataRange
    arg: DArgument

    def __init__(self,pred: DataRange,arg: DArgument,):
        ...
    

    ...
class DifferentIndividualsAtom:
    first: IArgument
    second: IArgument

    def __init__(self,first: IArgument,second: IArgument,):
        ...
    

    ...
class ObjectPropertyAtom:
    pred: ObjectPropertyExpression
    args: typing.Tuple[IArgument,IArgument]

    def __init__(self,pred: ObjectPropertyExpression,args: typing.Tuple[IArgument,IArgument],):
        ...
    

    ...
class SameIndividualAtom:
    first: IArgument
    second: IArgument

    def __init__(self,first: IArgument,second: IArgument,):
        ...
    

    ...

Atom = typing.Union[BuiltInAtom,ClassAtom,DataPropertyAtom,DataRangeAtom,DifferentIndividualsAtom,ObjectPropertyAtom,SameIndividualAtom,]

class Rule:
    head: typing.List[Atom]
    body: typing.List[Atom]

    def __init__(self,head: typing.List[Atom],body: typing.List[Atom],/):
        ...

    ...

Component = typing.Union[OntologyID,DocIRI,OntologyAnnotation,Import,DeclareClass,DeclareObjectProperty,DeclareAnnotationProperty,DeclareDataProperty,DeclareNamedIndividual,DeclareDatatype,SubClassOf,EquivalentClasses,DisjointClasses,DisjointUnion,SubObjectPropertyOf,EquivalentObjectProperties,DisjointObjectProperties,InverseObjectProperties,ObjectPropertyDomain,ObjectPropertyRange,FunctionalObjectProperty,InverseFunctionalObjectProperty,ReflexiveObjectProperty,IrreflexiveObjectProperty,SymmetricObjectProperty,AsymmetricObjectProperty,TransitiveObjectProperty,SubDataPropertyOf,EquivalentDataProperties,DisjointDataProperties,DataPropertyDomain,DataPropertyRange,FunctionalDataProperty,DatatypeDefinition,HasKey,SameIndividual,DifferentIndividuals,ClassAssertion,ObjectPropertyAssertion,NegativeObjectPropertyAssertion,DataPropertyAssertion,NegativeDataPropertyAssertion,AnnotationAssertion,SubAnnotationPropertyOf,AnnotationPropertyDomain,AnnotationPropertyRange,Rule,]

class AnnotatedComponent:
    component: Component
    ann: typing.Set[Annotation]

    def __init__(self,component: Component,ann: typing.Set[Annotation],/):
        ...

    ...




import typing
from typing import *
from typing_extensions import deprecated

import model

class PyIndexedOntology:
    """
    Represents a loaded ontology.
    """
    def get_id_for_iri(self, iri: str, iri_is_absolute: Optional[bool] = None) -> Optional[str]:
        """
        Gets the ID of term by it IRI.
        
        If the term does not have an ID, `None` is returned.
        """
        ...

    def get_iri_for_id(self, id: str) -> Optional[str]:
        """
        Gets the IRI of a term by its ID.
        
        If the term does not have an IRI, `None` is returned.
        """
        ...

    def add_prefix_mapping(self, iriprefix: str, mappedid: str) -> None:
        """
        Adds the prefix `iriprefix`.
        """
        ...

    def set_label(self, iri: str, label: str, *, absolute: Optional[bool] = None) -> None:
        """
        Sets the label of a term by iri.
        
        Adds an or updates the `AnnotationAssertion` axiom for `rdfs:label`.
        """
        ...

    def get_iri_for_label(self, label: str) -> Optional[str]:
        """
        Returns the IRI of a term by its label if it exists.
        
        If the term does not have a label, `None` is returned.
        """
        ...

    def get_iri(self) -> Optional[str]:
        """
        Returns the ontology iri, if it exists.
        """
        ...

    def get_version_iri(self) -> Optional[str]:
        """
        Returns the ontologys version iri, if it exists.
        """
        ...

    def get_subclasses(self, iri: str, iri_is_absolute: Optional[bool] = None) -> Set[str]:
        """
        Gets all subclasses of an entity.
        """
        ...

    def get_superclasses(self, iri: str, iri_is_absolute: Optional[bool] = None) -> Set[str]:
        """
        Gets all superclasses of an entity.
        """
        ...

    def get_classes(self) -> Set[str]:
        """
        Returns the IRIs of all declared classes in the ontology.
        """
        ...

    def get_object_properties(self) -> Set[str]:
        """
        Returns the IRIs of all declared object properties in the ontology.
        """
        ...

    def get_annotation(self, class_iri: str, ann_iri: str, *, class_iri_is_absolute: Optional[bool] = None, ann_iri_is_absolute: Optional[bool]=None) -> Optional[str]:
        """
        Gets the first annotated value for an entity and annotation property.
        
        Note: If there are multiple annotation axioms for the queried entity and annotation property,
        the order is neither necessarily the same as in the ontology neither is it stable.
        Get all annotation values with `PyIndexedOntology.get_annotations`.
        """
        ...

    def get_annotations(self, class_iri: str, ann_iri: str, *, class_iri_is_absolute: Optional[bool] = None, ann_iri_is_absolute: Optional[bool]=None) -> List[str]:
        """
        Gets all annotated value for an entity and annotation property.
        
        Note: The order is neither necessarily the same as in the ontology neither is it stable.
        Get all annotation values with `PyIndexedOntology.get_annotations`.
        """
        ...

    def save_to_string(self, serialization: typing.Literal['owl', 'rdf','ofn', 'owx']) -> str:
        """
        Saves the ontology to a UTF8 string.
        """
        ...

    def save_to_file(self, file_name: str, serialization: Optional[typing.Literal['owl', 'rdf','ofn', 'owx']]=None) -> None:
        """
        Saves the ontology to disk. If no serialization is given it is guessed by the file extension.
        Defaults to OWL/XML
        """
        ...

    def get_axioms_for_iri(self, iri: str, iri_is_absolute: Optional[bool] = None) -> List[model.AnnotatedComponent]:
        """
        Gets all axioms for an entity.
        """
        ...

    def get_components_for_iri(self, iri: str, iri_is_absolute: Optional[bool] = None) -> List[model.AnnotatedComponent]:
        """
        Gets all components (axiom, swrl, and meta component) for an entity.
        """
        ...

    def get_axioms(self) -> List[model.AnnotatedComponent]:
        """
        Returns all axioms of the ontology.
        """
        ...

    def get_components(self) -> List[model.AnnotatedComponent]:
        """
        Returns all axioms of the ontology.
        """
        ...

    def add_component(self, component: model.Component, annotations: Optional[Union[List[model.Annotation]|Set[model.Annotation]]]=None) -> None:
        """
        Adds an axiom to the ontology with optional annotations.
        """
        ...

    def add_axiom(self, ax: model.Component, annotations: Optional[Union[List[model.Annotation]|Set[model.Annotation]]]=None) -> None:
        """
        Synonym for `add_component`
        """
        ...

    def remove_component(self, component: model.Component) -> bool:
        """
        Removes a component from the ontology.
        """
        ...

    def remove_axiom(self, ax: model.Component) ->  bool:
        """
        Synonym for `remove_component`
        """
        ...

    def iri(self, iri: str, *, absolute: Optional[bool] = True) -> model.IRI:
        """
        Creates a new IRI from string.
        
        Use this method instead of  `model.IRI.parse` if possible as it is more optimized using caches.
        If `absolute` is None it is guessed by the occurrence of `"://"` in the IRI whether the iri
        is absolute or not.
        """
        ...

    def curie(self, iri: str) -> model.IRI:
        """
        Creates a new IRI from CURIE string.
        
        Use this method instead of  `model.IRI.parse` if possible as it is more optimized using caches.
        """
        ...

    def clazz(self, iri: str, *, absolute: Optional[bool]=None) -> model.Class:
        """
        Convenience method to create a Class from an IRI.
        
        Uses the `iri` method to cache native IRI instances.
        """
        ...

    def declare_class(self, iri: str, *, absolute: Optional[bool]=None) -> bool:
        """
        Convenience method to add a Declare(Class(iri)) axiom.
        """
        ...

    def object_property(self, iri: str, *, absolute: Optional[bool]=None) -> model.ObjectProperty:
        """
        Convenience method to create an ObjectProperty from an IRI.
        
        Uses the `iri` method to cache native IRI instances.
        """
        ...

    def declare_object_property(self, iri: str, *, absolute: Optional[bool]=None) -> bool:
        """
        Convenience method to add a Declare(ObjectProperty(iri)) axiom.
        """
        ...

    def data_property(self, iri: str, *, absolute: Optional[bool]=None) -> model.DataProperty:
        """
        Convenience method to create a DataProperty from an IRI.
        
        Uses the `iri` method to cache native IRI instances.
        """
        ...

    def declare_data_property(self, iri: str, *, absolute: Optional[bool]=None) -> bool:
        """
        Convenience method to add a Declare(DataProperty(iri)) axiom.
        """
        ...

    def annotation_property(self, iri: str, *, absolute: Optional[bool]=None) -> model.AnnotationProperty:
        """
        Convenience method to create an annotationProperty from an IRI.
        
        Uses the `iri` method to cache native IRI instances.
        """
        ...

    def declare_annotation_property(self, iri: str, *, absolute: Optional[bool]=None) -> bool:
        """
        Convenience method to add a Declare(annotationProperty(iri)) axiom.
        """
        ...

    def named_individual(self, iri: str, *, absolute: Optional[bool]=None) -> model.NamedIndividual:
        """
        Convenience method to create a NamedIndividual from an IRI.
        
        Uses the `iri` method to cache native IRI instances.
        """
        ...

    def declare_individual(self, iri: str, *, absolute: Optional[bool]=None) -> bool:
        """
        Convenience method to add a Declare(NamedIndividual(iri)) axiom.
        """
        ...

    def anonymous_individual(self, iri: str) -> model.AnonymousIndividual:
        """
        Convenience method to create an AnonymousIndividual from a string.
        """
        ...

    def get_descendants(self, parent: str, iri_is_absolute: Optional[bool] = None) -> Set[str]:
        """
        Gets all direct and indirect subclasses of a class.
        """
        ...

    def get_ancestors(self, child: str, iri_is_absolute: Optional[bool] = None) -> Set[str]:
        """
        Gets all direct and indirect super classes of a class.
        """
        ...

    def build_iri_index(self) -> None:
        """
        Builds an index by iri (IRIMappedIndex).
        """
        ...

    def component_index(self) -> None:
        """
        Builds an index by component kind (ComponentMappedIndex).
        """
        ...

    def build_indexes(self) -> None:
        """
        Builds indexes to allow (a quicker) access to axioms and entities.
        """
        ...

    prefix_mapping: PrefixMapping
    """
    The prefix mapping
    """


class IndexCreationStrategy:
    """
    Values to indicate when to build the additional indexes.
    
    """
    OnLoad: typing.Self
    """
    Create the additional indexes when the ontology is loaded
    """
    OnQuery: typing.Self
    """
    Create the additional indexes only when they are needed
    """
    Explicit: typing.Self
    """
    Only create the additional indexes when explicity requested
    """

class PrefixMapping:
    def __iter__(self, /):
        ...

    def __len__(self, /):
        ...

    def __getitem__(self, key, /):
        ...

    def __setitem__(self, key, value, /):
        ...

    def __delitem__(self, key, /):
        ...

    def __contains__(self, key, /):
        ...

    def add_default_prefix_names(self) -> None:
        """
        Adds the prefix for rdf, rdfs, xsd, and owl
        """
        ...

    def add_prefix(self, iriprefix: str, mappedid: str) -> None:
        """
        Adds the prefix `iriprefix`.
        """
        ...

    def remove_prefix(self, iriprefix: str) -> None:
        """
        Remove a prefix from the mapping.
        """
        ...

    def expand_curie(self, curie: str) -> str:
        """
        Expands a curie. Throws a ValueError if the prefix is invalid or unknown
        """
        ...

    def shrink_iri(self, iri: str) -> str:
        """
        Shrinks an absolute IRI to a CURIE. Throws a ValueError on failure
        """
        ...


def open_ontology(ontology: str, serialization: Optional[typing.Literal['owl', 'rdf','ofn', 'owx']]=None, index_strategy = IndexCreationStrategy.OnQuery) -> PyIndexedOntology:
    """
    Opens an ontology from a path or plain text.
    
    If `ontology` is a path, the file is loaded. Otherwise, `ontology` is interpreted as an ontology
    in plain text.
    If no serialization is specified the serialization is guessed by the file extension or all parsers are tried
    until one succeeds.
    """
    ...


def open_ontology_from_file(path: str, serialization: Optional[typing.Literal['owl', 'rdf','ofn', 'owx']]=None, index_strategy = IndexCreationStrategy.OnQuery) -> PyIndexedOntology:
    """
    Opens an ontology from a file
    
    If the serialization is not specified it is guessed from the file extension. Defaults to OWL/XML.
    """
    ...


def open_ontology_from_string(ontology: str, serialization: Optional[typing.Literal['owl', 'rdf','ofn', 'owx']]=None, index_strategy = IndexCreationStrategy.OnQuery) -> PyIndexedOntology:
    """
    Opens an ontology from plain text.
    
    If no serialization is specified, all parsers are tried until one succeeds
    """
    ...


@deprecated("please use `PyIndexedOntology.get_descendants` instead")
def get_descendants(onto: PyIndexedOntology, parent: str) -> Set[str]:
    """
    Gets all direct and indirect subclasses of a class.
    """
    ...


@deprecated("please use `PyIndexedOntology.get_ancestors` instead")
def get_ancestors(onto: PyIndexedOntology, child: str) -> Set[str]:
    """
    Gets all direct and indirect super classes of a class.
    """
    ...


def create_reasoner(name: str, ontology: PyIndexedOntology) -> PyReasoner:
    """
    Loads a reasoner from a shared library.
    
    :param str name: name of the reasoner to load or path to the shared library to load
    """
    ...


class PyReasoner:
    """
    A pyo3 compatible wrapper for all reasoner implementations
    """
    def get_name(self, /):
        ...

    def get_version(self, /):
        ...

    def is_consistent(self, /):
        ...

    def is_entailed(self, /, cmp):
        ...

    def is_satifisable(self, /, cmp):
        ...

    def get_unsatisfiable_classes(self, /):
        ...

    def get_subclasses(self, /, cmp):
        ...

    def get_superclasses(self, /, cmp):
        ...

    def get_equivalent_classes(self, /, cmp):
        ...

    def get_disjoint_classes(self, /, cmp):
        ...

    def get_current_ontology(self, /):
        ...

    def add_component(self, component: model.Component, annotations: Optional[Union[List[model.Annotation]|Set[model.Annotation]]]=None) -> None:
        """
        Adds an axiom to the ontology with optional annotations.
        """
        ...

    def remove_component(self, component: model.Component) -> bool:
        """
        Removes a component from the ontology.
        """
        ...


